unit main;

interface

uses
  Windows, SysUtils, Controls, Classes, Forms, Dialogs,
  StdCtrls, FileCtrl, IniFiles, Menus, Clipbrd;

type
  TMainForm = class(TForm)
    DirEdit: TEdit;
    Label1: TLabel;
    SelectButton: TButton;
    EasyEdit: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    HardEdit: TEdit;
    DisableButton: TButton;
    LogCheckBox: TCheckBox;
    procedure SelectButtonClick(Sender: TObject);
    procedure DisableButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private 錾 }
  public
    IniFile: TIniFile;
    procedure DisableAllButton;
    procedure EnableAllButton;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.SelectButtonClick(Sender: TObject);
var
  S: string;
begin
  if SelectDirectory('BM98fBNg̑I', '', S) then
    DirEdit.Text := S;
end;

procedure TMainForm.DisableButtonClick(Sender: TObject);
var
  ClearData: TStringList;
  I: Integer;
  Cleared, Sum, Disabled: array[0..16] of Integer;
  S: string;
  HardLine, EasyLine: Integer;

  function GetLevel(const FileName: TFileName): Integer;
  var
    F: TextFile;
    S: string;
  begin
    AssignFile(F, FileName);
    Reset(F);
    Result := -1;

    while not EoF(F) do
    begin
      ReadLn(F, S);
      if AnsiLowerCase(Copy(S, 1, 10)) = '#playlevel' then
      begin
        Result := StrToIntDef(Copy(S, 11, Length(S)), 0);
        Break;
      end;
    end;

    CloseFile(F);
  end;

  procedure DisableBMS(Dir: TFileName);
  var
    Handle: THandle;
    Data: TWin32FindData;
    Level: Integer;
    Clear: Boolean;
    Index: Integer; //not use
    FileName: TFileName;
  begin
    Handle := Windows.FindFirstFile(PChar(Dir + '\*.*'), Data);
    if Handle = INVALID_HANDLE_VALUE then Exit;

    if not ((StrPas(Data.cFileName) = '.') or (StrPas(Data.cFileName) = '..')) then
    begin
      if (Data.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) <> 0 then
        DisableBMS(Dir + '\' + StrPas(Data.cFileName));
    end;

    while Windows.FindNextFile(Handle, Data) = True do
    begin
      FileName := AnsiLowerCase(StrPas(Data.cFileName));
      if not ((FileName = '.') or (FileName = '..')) then
      begin
        if ExtractFileExt(FileName) = '.$$$' then
        begin
          RenameFile(Dir + '\' + FileName, Dir + '\' + ChangeFileExt(FileName, '.bms'));
          FileName := ChangeFileExt(FileName, '.bms');
        end;

        if ExtractFileExt(FileName) = '.bms' then
        begin
          Level := GetLevel(Dir + '\' + FileName);

          if (Level > -1) and (Level < 17) then
          begin
            Inc(Sum[Level]);

            Clear := ClearData.Find(Dir + '\' + FileName, Index);
            if Clear then Inc(Cleared[Level]);

            if (Clear       and (Level <= EasyLine)) or
               ((not Clear) and (Level >= HardLine)) then
            begin
              RenameFile(Dir + '\' + FileName, Dir + '\' + ChangeFileExt(FileName, '.$$$'));
              Inc(Disabled[Level]);
            end;
          end;
        end;

        if (Data.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) <> 0 then
          DisableBMS(Dir + '\' + FileName);
      end;
    end;

    Windows.FindClose(Handle);
  end;

begin
  if not DirectoryExists(DirEdit.Text) then
  begin
    ShowMessage('w肳ꂽfBNg݂͑܂');
    Exit;
  end;

  if not FileExists(DirEdit.Text + '\clear.dat') then
  begin
    ShowMessage(DirEdit.Text + '\clear.dat' + '݂܂');
    Exit;
  end;

  ClearData := TStringList.Create;
  ClearData.LoadFromFile(DirEdit.Text + '\clear.dat');
  ClearData.Sort;

  for I := 0 to 16 do
  begin
    Cleared[I] := 0;
    Sum[I] := 0;
    Disabled[I] := 0;
  end;

  for I := 1 to ClearData.Count - 1 do
  begin
    ClearData.Strings[I] := DirEdit.Text + '\' + ClearData.Strings[I];
  end;

  EasyLine := StrToIntDef(EasyEdit.Text, 16);
  HardLine := StrToIntDef(HardEdit.Text, 1);

  DisableAllButton;
  DisableBMS(DirEdit.Text);
  EnableAllButton;

  if LogCheckBox.Checked then
  begin
    S := 'Level'#9'Sum'#9'Cleared'#9'Disabled'#13#10;
    for I := 0 to 16 do
    begin
      S := S + IntToStr(I) + #9;
      S := S + IntToStr(Sum[I]) + #9;
      S := S + IntToStr(Cleared[I]) + #9;
      S := S + IntToStr(Disabled[I]) + #13#10;
    end;
    Clipboard.AsText := S;
  end;

  ClearData.Free;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  IniFile := TIniFile.Create(ChangeFileExt(AnsiLowerCase(Application.ExeName), '.ini'));
  DirEdit.Text := IniFile.ReadString('dir', 'bm98dir', '');
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  IniFile.WriteString('dir', 'bm98dir', DirEdit.Text);
  IniFile.Free;
end;

procedure TMainForm.DisableAllButton;
begin
  SelectButton.Enabled := False;
  DisableButton.Enabled := False;
end;

procedure TMainForm.EnableAllButton;
begin
  SelectButton.Enabled := True;
  DisableButton.Enabled := True;
end;

end.
